#!/usr/bin/env python
# Copyright 2010 Google Inc.
# Licensed under the Apache License, Version 2.0
# http://www.apache.org/licenses/LICENSE-2.0

# Google's Python Class
# http://code.google.com/edu/languages/google-python-class/

from __future__ import print_function

"""Wordcount exercise
Google's Python class

The main() below is already defined and complete. It calls print_words()
and print_top() functions which you write.

1. For the --count flag, implement a print_words(filename) function that counts
how often each word appears in the text and prints:
word1 count1
word2 count2
...

Print the above list in order sorted by word (python will sort punctuation to
come before letters -- that's fine). Store all the words as lowercase,
so 'The' and 'the' count as the same word.

2. For the --topcount flag, implement a print_top(filename) which is similar
to print_words() but which prints just the top 20 most common words sorted
so the most common word is first, then the next most common, and so on.

Use str.split() (no arguments) to split on all whitespace.

Workflow: don't build the whole program at once. Get it to an intermediate
milestone and print your data structure and sys.exit(0).
When that's working, try for the next milestone.

Optional: define a helper function to avoid code duplication inside
print_words() and print_top().

"""

import sys
import re

# +++your code here+++
# Define print_words(filename) and print_top(filename) functions.
# You could write a helper utility function that reads a file
# and builds and returns a word/count dict for it.
# Then print_words() and print_top() can just call the utility function.

###

def make_word_dict(string):
    word_list = string.split()
    mydict = {}
    for word in word_list:
        downcase_word = re.sub('[^a-z]','',word.lower()) # lowercase, no punctuation
        if mydict.get(downcase_word, None) == None:
            mydict[downcase_word] = 1
        else:
            mydict[downcase_word] += 1
    return mydict

def print_words(word_dict):
    for word in sorted(word_dict):
        print("{0:15}{1:4d}".format(word, word_dict[word]))

def print_top(word_dict, n):
    sorted_word_dict = sorted(word_dict, reverse=True, key = lambda key: word_dict[key])
    cnt = 0 if n == 'all' else n
    for key in sorted_word_dict:
        print("{0:15}{1:4d}".format(key, word_dict[key]))
        cnt -=1
        if cnt == 0:
            return


def read_file_into_string(filename):
    try:
        with open(filename, 'r') as f:
            return(f.read())
    except:
        print("Error: could not open {0}.".format(filename))
        return None
    

# This basic command line argument parsing code is provided and
# calls the print_words() and print_top() functions which you must define.
def main():
    if len(sys.argv) != 3:
        print('usage: ./wordcount.py {--count | --topcount | --allcount} file')
        sys.exit(1)
    option = sys.argv[1]
    filename = sys.argv[2]
    file_contents_raw = read_file_into_string(filename)
    if not file_contents_raw:
        sys.exit(1)
    file_contents = re.sub('--', ' ', file_contents_raw)
    word_dict = make_word_dict(file_contents)
    if option == '--count':
        print_words(word_dict)
    elif option == '--topcount':
        print_top(word_dict, 20)
    elif option == '--allcount':
        print_top(word_dict, 'all')
    else:
        print('unknown option: ' + option)
        sys.exit(1)

if __name__ == '__main__':
  main()
  
