#!/usr/bin/python -tt
# Copyright 2010 Google Inc.
# Licensed under the Apache License, Version 2.0
# http://www.apache.org/licenses/LICENSE-2.0

# Google's Python Class
# http://code.google.com/edu/languages/google-python-class/

"""Mimic pyquick exercise -- optional extra exercise.
Google's Python Class

Read in the file specified on the command line.
Do a simple split() on whitespace to obtain all the words in the file.
Rather than read the file line by line, it's easier to read
it into one giant string and split it once.

Build a "mimic" dict that maps each word that appears in the file
to a list of all the words that immediately follow that word in the file.
The list of words can be be in any order and should include
duplicates. So for example the key "and" might have the list
["then", "best", "then", "after", ...] listing
all the words which came after "and" in the text.
We'll say that the empty string is what comes before
the first word in the file.

With the mimic dict, it's fairly easy to emit random
text that mimics the original. Print a word, then look
up what words might come next and pick one at random as
the next work.
Use the empty string as the first word to prime things.
If we ever get stuck with a word that is not in the dict,
go back to the empty string to keep things moving.

Note: the standard python module 'random' includes a
random.choice(list) method which picks a random element
from a non-empty list.

For fun, feed your program to itself as input.
Could work on getting it to put in linebreaks around 70
columns, so the output looks better.

"""

import random
import sys
import string

def mimic_dict(filename):
  """Returns mimic dict mapping each word to list of words which follow it."""
  md = {}
  with open(filename, 'r') as f:
      all_words_list = f.read().translate(None, string.punctuation).lower().split()
  previous_word = ''
  for word in all_words_list:
      if previous_word in md:
          md[previous_word].append(word)
      else:
          md[previous_word] = [word]
      previous_word = word
  return md


def print_mimic(mimic_dict, word, n):
  """Given mimic dict and start word, prints n random words."""
  for i in range(n):
      print word,
      if word in mimic_dict:
          word = random.choice(mimic_dict[word])
      else:
          word = mimic_dict[''][0] # if word not found, default to first word in dict
# or
#      word = random.choice(mimic_dict[word]) if word in mimic_dict else mimic_dict[''][0]


# Provided main(), calls mimic_dict() and mimic()
def main():
  if len(sys.argv) != 3:
    print 'usage: ./mimic.py file-to-read n'
    sys.exit(1)

  d = mimic_dict(sys.argv[1])
  n = int(sys.argv[2])
  print_mimic(d, '', n)


if __name__ == '__main__':
  main()
